# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .batch_test_response import BatchTestResponse

__all__ = ["TestListBatchTestsResponse"]

TestListBatchTestsResponse: TypeAlias = List[BatchTestResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ..types import voice_clone_params, voice_search_params, voice_add_resource_params
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, SequenceNotStr, omit, not_given
from .._utils import extract_files, path_template, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.voice_response import VoiceResponse
from ..types.voice_list_response import VoiceListResponse
from ..types.voice_search_response import VoiceSearchResponse

__all__ = ["VoiceResource", "AsyncVoiceResource"]


class VoiceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return VoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return VoiceResourceWithStreamingResponse(self)

    def retrieve(
        self,
        voice_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceResponse:
        """
        Retrieve details of a specific voice

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not voice_id:
            raise ValueError(f"Expected a non-empty value for `voice_id` but received {voice_id!r}")
        return self._get(
            path_template("/get-voice/{voice_id}", voice_id=voice_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceListResponse:
        """List all voices available to the user"""
        return self._get(
            "/list-voices",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceListResponse,
        )

    def add_resource(
        self,
        *,
        provider_voice_id: str,
        voice_name: str,
        public_user_id: str | Omit = omit,
        voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceResponse:
        """
        Add a community voice to the voice library

        Args:
          provider_voice_id: Voice id assigned by the provider.

          voice_name: A custom name for the voice.

          public_user_id: Required for ElevenLabs only. User id of the voice owner.

          voice_provider: Voice provider to add the voice from.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/add-community-voice",
            body=maybe_transform(
                {
                    "provider_voice_id": provider_voice_id,
                    "voice_name": voice_name,
                    "public_user_id": public_user_id,
                    "voice_provider": voice_provider,
                },
                voice_add_resource_params.VoiceAddResourceParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceResponse,
        )

    def clone(
        self,
        *,
        files: SequenceNotStr[FileTypes],
        voice_name: str,
        voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio", "platform"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceResponse:
        """
        Clone a voice from audio files

        Args:
          files: Audio files to use for voice cloning. Up to 25 files allowed. For Cartesia and
              MiniMax, only 1 file is supported. For Inworld, up to 3 files are supported.

          voice_name: Name for the cloned voice

          voice_provider: Voice provider to use for cloning.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "files": files,
                "voice_name": voice_name,
                "voice_provider": voice_provider,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/clone-voice",
            body=maybe_transform(body, voice_clone_params.VoiceCloneParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceResponse,
        )

    def search(
        self,
        *,
        search_query: str,
        voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceSearchResponse:
        """
        Search for community voices from voice providers

        Args:
          search_query: Search query to find voices by name, description, or ID.

          voice_provider: Voice provider to search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/search-community-voice",
            body=maybe_transform(
                {
                    "search_query": search_query,
                    "voice_provider": voice_provider,
                },
                voice_search_params.VoiceSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceSearchResponse,
        )


class AsyncVoiceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncVoiceResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        voice_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceResponse:
        """
        Retrieve details of a specific voice

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not voice_id:
            raise ValueError(f"Expected a non-empty value for `voice_id` but received {voice_id!r}")
        return await self._get(
            path_template("/get-voice/{voice_id}", voice_id=voice_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceListResponse:
        """List all voices available to the user"""
        return await self._get(
            "/list-voices",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceListResponse,
        )

    async def add_resource(
        self,
        *,
        provider_voice_id: str,
        voice_name: str,
        public_user_id: str | Omit = omit,
        voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceResponse:
        """
        Add a community voice to the voice library

        Args:
          provider_voice_id: Voice id assigned by the provider.

          voice_name: A custom name for the voice.

          public_user_id: Required for ElevenLabs only. User id of the voice owner.

          voice_provider: Voice provider to add the voice from.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/add-community-voice",
            body=await async_maybe_transform(
                {
                    "provider_voice_id": provider_voice_id,
                    "voice_name": voice_name,
                    "public_user_id": public_user_id,
                    "voice_provider": voice_provider,
                },
                voice_add_resource_params.VoiceAddResourceParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceResponse,
        )

    async def clone(
        self,
        *,
        files: SequenceNotStr[FileTypes],
        voice_name: str,
        voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio", "platform"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceResponse:
        """
        Clone a voice from audio files

        Args:
          files: Audio files to use for voice cloning. Up to 25 files allowed. For Cartesia and
              MiniMax, only 1 file is supported. For Inworld, up to 3 files are supported.

          voice_name: Name for the cloned voice

          voice_provider: Voice provider to use for cloning.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "files": files,
                "voice_name": voice_name,
                "voice_provider": voice_provider,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/clone-voice",
            body=await async_maybe_transform(body, voice_clone_params.VoiceCloneParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceResponse,
        )

    async def search(
        self,
        *,
        search_query: str,
        voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceSearchResponse:
        """
        Search for community voices from voice providers

        Args:
          search_query: Search query to find voices by name, description, or ID.

          voice_provider: Voice provider to search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/search-community-voice",
            body=await async_maybe_transform(
                {
                    "search_query": search_query,
                    "voice_provider": voice_provider,
                },
                voice_search_params.VoiceSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceSearchResponse,
        )


class VoiceResourceWithRawResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.retrieve = to_raw_response_wrapper(
            voice.retrieve,
        )
        self.list = to_raw_response_wrapper(
            voice.list,
        )
        self.add_resource = to_raw_response_wrapper(
            voice.add_resource,
        )
        self.clone = to_raw_response_wrapper(
            voice.clone,
        )
        self.search = to_raw_response_wrapper(
            voice.search,
        )


class AsyncVoiceResourceWithRawResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.retrieve = async_to_raw_response_wrapper(
            voice.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            voice.list,
        )
        self.add_resource = async_to_raw_response_wrapper(
            voice.add_resource,
        )
        self.clone = async_to_raw_response_wrapper(
            voice.clone,
        )
        self.search = async_to_raw_response_wrapper(
            voice.search,
        )


class VoiceResourceWithStreamingResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.retrieve = to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            voice.list,
        )
        self.add_resource = to_streamed_response_wrapper(
            voice.add_resource,
        )
        self.clone = to_streamed_response_wrapper(
            voice.clone,
        )
        self.search = to_streamed_response_wrapper(
            voice.search,
        )


class AsyncVoiceResourceWithStreamingResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.retrieve = async_to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            voice.list,
        )
        self.add_resource = async_to_streamed_response_wrapper(
            voice.add_resource,
        )
        self.clone = async_to_streamed_response_wrapper(
            voice.clone,
        )
        self.search = async_to_streamed_response_wrapper(
            voice.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import mcp_tool_get_mcp_tools_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.mcp_tool_get_mcp_tools_response import McpToolGetMcpToolsResponse

__all__ = ["McpToolResource", "AsyncMcpToolResource"]


class McpToolResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> McpToolResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return McpToolResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> McpToolResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return McpToolResourceWithStreamingResponse(self)

    def get_mcp_tools(
        self,
        agent_id: str,
        *,
        mcp_id: str,
        component_id: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpToolGetMcpToolsResponse:
        """
        Get MCP tools for a specific agent

        Args:
          mcp_id: The ID of the MCP server to get tools from.

          component_id: The ID of the component if the MCP server is configured under a component.

          version: Optional version of the agent to use for this request. Default to latest
              version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get(
            path_template("/get-mcp-tools/{agent_id}", agent_id=agent_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "mcp_id": mcp_id,
                        "component_id": component_id,
                        "version": version,
                    },
                    mcp_tool_get_mcp_tools_params.McpToolGetMcpToolsParams,
                ),
            ),
            cast_to=McpToolGetMcpToolsResponse,
        )


class AsyncMcpToolResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMcpToolResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncMcpToolResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMcpToolResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/RetellAI/retell-python-sdk#with_streaming_response
        """
        return AsyncMcpToolResourceWithStreamingResponse(self)

    async def get_mcp_tools(
        self,
        agent_id: str,
        *,
        mcp_id: str,
        component_id: str | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpToolGetMcpToolsResponse:
        """
        Get MCP tools for a specific agent

        Args:
          mcp_id: The ID of the MCP server to get tools from.

          component_id: The ID of the component if the MCP server is configured under a component.

          version: Optional version of the agent to use for this request. Default to latest
              version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._get(
            path_template("/get-mcp-tools/{agent_id}", agent_id=agent_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "mcp_id": mcp_id,
                        "component_id": component_id,
                        "version": version,
                    },
                    mcp_tool_get_mcp_tools_params.McpToolGetMcpToolsParams,
                ),
            ),
            cast_to=McpToolGetMcpToolsResponse,
        )


class McpToolResourceWithRawResponse:
    def __init__(self, mcp_tool: McpToolResource) -> None:
        self._mcp_tool = mcp_tool

        self.get_mcp_tools = to_raw_response_wrapper(
            mcp_tool.get_mcp_tools,
        )


class AsyncMcpToolResourceWithRawResponse:
    def __init__(self, mcp_tool: AsyncMcpToolResource) -> None:
        self._mcp_tool = mcp_tool

        self.get_mcp_tools = async_to_raw_response_wrapper(
            mcp_tool.get_mcp_tools,
        )


class McpToolResourceWithStreamingResponse:
    def __init__(self, mcp_tool: McpToolResource) -> None:
        self._mcp_tool = mcp_tool

        self.get_mcp_tools = to_streamed_response_wrapper(
            mcp_tool.get_mcp_tools,
        )


class AsyncMcpToolResourceWithStreamingResponse:
    def __init__(self, mcp_tool: AsyncMcpToolResource) -> None:
        self._mcp_tool = mcp_tool

        self.get_mcp_tools = async_to_streamed_response_wrapper(
            mcp_tool.get_mcp_tools,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VoiceSearchParams"]


class VoiceSearchParams(TypedDict, total=False):
    search_query: Required[str]
    """Search query to find voices by name, description, or ID."""

    voice_provider: Literal["elevenlabs", "cartesia", "minimax", "fish_audio"]
    """Voice provider to search."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["VoiceResponse"]


class VoiceResponse(BaseModel):
    gender: Literal["male", "female"]
    """Gender of voice."""

    provider: Literal["elevenlabs", "openai", "deepgram", "cartesia", "minimax", "fish_audio", "platform"]
    """Indicates the provider of voice."""

    voice_id: str
    """Unique id for the voice."""

    voice_name: str
    """Name of the voice."""

    accent: Optional[str] = None
    """Accent annotation of the voice."""

    age: Optional[str] = None
    """Age annotation of the voice."""

    preview_audio_url: Optional[str] = None
    """URL to the preview audio of the voice."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .llm_response import LlmResponse as LlmResponse
from .call_response import CallResponse as CallResponse
from .chat_response import ChatResponse as ChatResponse
from .agent_response import AgentResponse as AgentResponse
from .voice_response import VoiceResponse as VoiceResponse
from .llm_list_params import LlmListParams as LlmListParams
from .call_list_params import CallListParams as CallListParams
from .chat_list_params import ChatListParams as ChatListParams
from .agent_list_params import AgentListParams as AgentListParams
from .llm_create_params import LlmCreateParams as LlmCreateParams
from .llm_list_response import LlmListResponse as LlmListResponse
from .llm_update_params import LlmUpdateParams as LlmUpdateParams
from .web_call_response import WebCallResponse as WebCallResponse
from .call_list_response import CallListResponse as CallListResponse
from .call_update_params import CallUpdateParams as CallUpdateParams
from .chat_create_params import ChatCreateParams as ChatCreateParams
from .chat_list_response import ChatListResponse as ChatListResponse
from .chat_update_params import ChatUpdateParams as ChatUpdateParams
from .voice_clone_params import VoiceCloneParams as VoiceCloneParams
from .agent_create_params import AgentCreateParams as AgentCreateParams
from .agent_list_response import AgentListResponse as AgentListResponse
from .agent_update_params import AgentUpdateParams as AgentUpdateParams
from .batch_call_response import BatchCallResponse as BatchCallResponse
from .batch_test_response import BatchTestResponse as BatchTestResponse
from .chat_agent_response import ChatAgentResponse as ChatAgentResponse
from .llm_retrieve_params import LlmRetrieveParams as LlmRetrieveParams
from .mcp_tool_definition import McpToolDefinition as McpToolDefinition
from .phone_call_response import PhoneCallResponse as PhoneCallResponse
from .voice_list_response import VoiceListResponse as VoiceListResponse
from .voice_search_params import VoiceSearchParams as VoiceSearchParams
from .agent_retrieve_params import AgentRetrieveParams as AgentRetrieveParams
from .phone_number_response import PhoneNumberResponse as PhoneNumberResponse
from .voice_search_response import VoiceSearchResponse as VoiceSearchResponse
from .chat_agent_list_params import ChatAgentListParams as ChatAgentListParams
from .test_case_job_response import TestCaseJobResponse as TestCaseJobResponse
from .knowledge_base_response import KnowledgeBaseResponse as KnowledgeBaseResponse
from .chat_agent_create_params import ChatAgentCreateParams as ChatAgentCreateParams
from .chat_agent_list_response import ChatAgentListResponse as ChatAgentListResponse
from .chat_agent_update_params import ChatAgentUpdateParams as ChatAgentUpdateParams
from .voice_add_resource_params import VoiceAddResourceParams as VoiceAddResourceParams
from .chat_agent_retrieve_params import ChatAgentRetrieveParams as ChatAgentRetrieveParams
from .conversation_flow_response import ConversationFlowResponse as ConversationFlowResponse
from .phone_number_create_params import PhoneNumberCreateParams as PhoneNumberCreateParams
from .phone_number_import_params import PhoneNumberImportParams as PhoneNumberImportParams
from .phone_number_list_response import PhoneNumberListResponse as PhoneNumberListResponse
from .phone_number_update_params import PhoneNumberUpdateParams as PhoneNumberUpdateParams
from .agent_get_versions_response import AgentGetVersionsResponse as AgentGetVersionsResponse
from .call_create_web_call_params import CallCreateWebCallParams as CallCreateWebCallParams
from .chat_create_sms_chat_params import ChatCreateSMSChatParams as ChatCreateSMSChatParams
from .knowledge_base_create_params import KnowledgeBaseCreateParams as KnowledgeBaseCreateParams
from .knowledge_base_list_response import KnowledgeBaseListResponse as KnowledgeBaseListResponse
from .test_list_batch_tests_params import TestListBatchTestsParams as TestListBatchTestsParams
from .test_list_test_runs_response import TestListTestRunsResponse as TestListTestRunsResponse
from .call_create_phone_call_params import CallCreatePhoneCallParams as CallCreatePhoneCallParams
from .concurrency_retrieve_response import ConcurrencyRetrieveResponse as ConcurrencyRetrieveResponse
from .conversation_flow_list_params import ConversationFlowListParams as ConversationFlowListParams
from .mcp_tool_get_mcp_tools_params import McpToolGetMcpToolsParams as McpToolGetMcpToolsParams
from .test_case_definition_response import TestCaseDefinitionResponse as TestCaseDefinitionResponse
from .test_create_batch_test_params import TestCreateBatchTestParams as TestCreateBatchTestParams
from .test_list_batch_tests_response import TestListBatchTestsResponse as TestListBatchTestsResponse
from .call_register_phone_call_params import CallRegisterPhoneCallParams as CallRegisterPhoneCallParams
from .conversation_flow_create_params import ConversationFlowCreateParams as ConversationFlowCreateParams
from .conversation_flow_list_response import ConversationFlowListResponse as ConversationFlowListResponse
from .conversation_flow_update_params import ConversationFlowUpdateParams as ConversationFlowUpdateParams
from .mcp_tool_get_mcp_tools_response import McpToolGetMcpToolsResponse as McpToolGetMcpToolsResponse
from .chat_agent_get_versions_response import ChatAgentGetVersionsResponse as ChatAgentGetVersionsResponse
from .conversation_flow_retrieve_params import ConversationFlowRetrieveParams as ConversationFlowRetrieveParams
from .knowledge_base_add_sources_params import KnowledgeBaseAddSourcesParams as KnowledgeBaseAddSourcesParams
from .chat_create_chat_completion_params import ChatCreateChatCompletionParams as ChatCreateChatCompletionParams
from .batch_call_create_batch_call_params import BatchCallCreateBatchCallParams as BatchCallCreateBatchCallParams
from .chat_create_chat_completion_response import ChatCreateChatCompletionResponse as ChatCreateChatCompletionResponse
from .conversation_flow_component_response import ConversationFlowComponentResponse as ConversationFlowComponentResponse
from .test_list_test_case_definitions_params import (
    TestListTestCaseDefinitionsParams as TestListTestCaseDefinitionsParams,
)
from .test_create_test_case_definition_params import (
    TestCreateTestCaseDefinitionParams as TestCreateTestCaseDefinitionParams,
)
from .test_update_test_case_definition_params import (
    TestUpdateTestCaseDefinitionParams as TestUpdateTestCaseDefinitionParams,
)
from .test_list_test_case_definitions_response import (
    TestListTestCaseDefinitionsResponse as TestListTestCaseDefinitionsResponse,
)
from .conversation_flow_component_create_params import (
    ConversationFlowComponentCreateParams as ConversationFlowComponentCreateParams,
)
from .conversation_flow_component_list_response import (
    ConversationFlowComponentListResponse as ConversationFlowComponentListResponse,
)
from .conversation_flow_component_update_params import (
    ConversationFlowComponentUpdateParams as ConversationFlowComponentUpdateParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "TestCaseDefinitionResponse",
    "ResponseEngine",
    "ResponseEngineResponseEngineRetellLm",
    "ResponseEngineResponseEngineConversationFlow",
    "ToolMock",
    "ToolMockInputMatchRule",
    "ToolMockInputMatchRuleType",
    "ToolMockInputMatchRuleUnionMember1",
]


class ResponseEngineResponseEngineRetellLm(BaseModel):
    llm_id: str
    """id of the Retell LLM Response Engine."""

    type: Literal["retell-llm"]
    """type of the Response Engine."""

    version: Optional[float] = None
    """Version of the Retell LLM Response Engine."""


class ResponseEngineResponseEngineConversationFlow(BaseModel):
    conversation_flow_id: str
    """ID of the Conversation Flow Response Engine."""

    type: Literal["conversation-flow"]
    """type of the Response Engine."""

    version: Optional[float] = None
    """Version of the Conversation Flow Response Engine."""


ResponseEngine: TypeAlias = Union[ResponseEngineResponseEngineRetellLm, ResponseEngineResponseEngineConversationFlow]


class ToolMockInputMatchRuleType(BaseModel):
    type: Literal["any"]
    """Match any input of the tool"""


class ToolMockInputMatchRuleUnionMember1(BaseModel):
    args: Dict[str, object]
    """Arguments to match. Only provided fields will be checked"""

    type: Literal["partial_match"]
    """Match only calls with specific arguments"""


ToolMockInputMatchRule: TypeAlias = Union[ToolMockInputMatchRuleType, ToolMockInputMatchRuleUnionMember1]


class ToolMock(BaseModel):
    input_match_rule: ToolMockInputMatchRule
    """Rule for matching tool calls"""

    output: str
    """The output of the tool call that will be fed into the LLM.

    Should be a JSON string.
    """

    tool_name: str
    """Name of the tool to mock"""

    result: Optional[bool] = None
    """For tool calls like transfer_call that require a boolean result.

    Optional for most tools.
    """


class TestCaseDefinitionResponse(BaseModel):
    __test__ = False
    creation_timestamp: int
    """Timestamp when the test case definition was created (milliseconds since epoch)"""

    dynamic_variables: Dict[str, str]
    """Dynamic variables to inject into the response engine"""

    llm_model: Literal[
        "gpt-4.1",
        "gpt-4.1-mini",
        "gpt-4.1-nano",
        "gpt-5",
        "gpt-5-mini",
        "gpt-5-nano",
        "gpt-5.1",
        "gpt-5.2",
        "gpt-5.4",
        "gpt-5.4-mini",
        "gpt-5.4-nano",
        "claude-4.5-sonnet",
        "claude-4.6-sonnet",
        "claude-4.5-haiku",
        "gemini-2.5-flash",
        "gemini-2.5-flash-lite",
        "gemini-3.0-flash",
    ]
    """LLM model to use for simulation"""

    metrics: List[str]
    """Array of metric names to evaluate"""

    name: str
    """Name of the test case definition"""

    response_engine: ResponseEngine
    """Response engine to use for the test case. Custom LLM is not supported."""

    test_case_definition_id: str
    """Unique identifier for the test case definition"""

    tool_mocks: List[ToolMock]
    """Mock tool calls for testing"""

    type: Literal["simulation"]
    """Type of test case definition"""

    user_modified_timestamp: int
    """
    Timestamp when the test case definition was last modified (milliseconds since
    epoch)
    """

    user_prompt: str
    """User prompt to simulate in the test case"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import (
    BatchTestResponse,
    TestCaseJobResponse,
    TestListTestRunsResponse,
    TestCaseDefinitionResponse,
    TestListBatchTestsResponse,
    TestListTestCaseDefinitionsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTests:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_batch_test(self, client: Retell) -> None:
        test = client.tests.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            test_case_definition_ids=["string"],
        )
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_batch_test_with_all_params(self, client: Retell) -> None:
        test = client.tests.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
                "version": 0,
            },
            test_case_definition_ids=["string"],
        )
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_batch_test(self, client: Retell) -> None:
        response = client.tests.with_raw_response.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            test_case_definition_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_batch_test(self, client: Retell) -> None:
        with client.tests.with_streaming_response.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            test_case_definition_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(BatchTestResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_test_case_definition(self, client: Retell) -> None:
        test = client.tests.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            user_prompt="user_prompt",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_test_case_definition_with_all_params(self, client: Retell) -> None:
        test = client.tests.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
                "version": 0,
            },
            user_prompt="user_prompt",
            dynamic_variables={"foo": "string"},
            llm_model="gpt-4.1",
            tool_mocks=[
                {
                    "input_match_rule": {"type": "any"},
                    "output": "output",
                    "tool_name": "tool_name",
                    "result": True,
                }
            ],
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_test_case_definition(self, client: Retell) -> None:
        response = client.tests.with_raw_response.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            user_prompt="user_prompt",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_test_case_definition(self, client: Retell) -> None:
        with client.tests.with_streaming_response.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            user_prompt="user_prompt",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete_test_case_definition(self, client: Retell) -> None:
        test = client.tests.delete_test_case_definition(
            "test_case_definition_id",
        )
        assert test is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete_test_case_definition(self, client: Retell) -> None:
        response = client.tests.with_raw_response.delete_test_case_definition(
            "test_case_definition_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert test is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete_test_case_definition(self, client: Retell) -> None:
        with client.tests.with_streaming_response.delete_test_case_definition(
            "test_case_definition_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert test is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete_test_case_definition(self, client: Retell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_definition_id` but received ''"
        ):
            client.tests.with_raw_response.delete_test_case_definition(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_batch_test(self, client: Retell) -> None:
        test = client.tests.get_batch_test(
            "test_case_batch_job_id",
        )
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_batch_test(self, client: Retell) -> None:
        response = client.tests.with_raw_response.get_batch_test(
            "test_case_batch_job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_batch_test(self, client: Retell) -> None:
        with client.tests.with_streaming_response.get_batch_test(
            "test_case_batch_job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(BatchTestResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_batch_test(self, client: Retell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_batch_job_id` but received ''"
        ):
            client.tests.with_raw_response.get_batch_test(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_test_case_definition(self, client: Retell) -> None:
        test = client.tests.get_test_case_definition(
            "test_case_definition_id",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_test_case_definition(self, client: Retell) -> None:
        response = client.tests.with_raw_response.get_test_case_definition(
            "test_case_definition_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_test_case_definition(self, client: Retell) -> None:
        with client.tests.with_streaming_response.get_test_case_definition(
            "test_case_definition_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_test_case_definition(self, client: Retell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_definition_id` but received ''"
        ):
            client.tests.with_raw_response.get_test_case_definition(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_test_run(self, client: Retell) -> None:
        test = client.tests.get_test_run(
            "test_case_job_id",
        )
        assert_matches_type(TestCaseJobResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_test_run(self, client: Retell) -> None:
        response = client.tests.with_raw_response.get_test_run(
            "test_case_job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestCaseJobResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_test_run(self, client: Retell) -> None:
        with client.tests.with_streaming_response.get_test_run(
            "test_case_job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestCaseJobResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_test_run(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_job_id` but received ''"):
            client.tests.with_raw_response.get_test_run(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_batch_tests(self, client: Retell) -> None:
        test = client.tests.list_batch_tests(
            type="retell-llm",
        )
        assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_batch_tests_with_all_params(self, client: Retell) -> None:
        test = client.tests.list_batch_tests(
            type="retell-llm",
            conversation_flow_id="conversation_flow_id",
            llm_id="llm_id",
            version=0,
        )
        assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_batch_tests(self, client: Retell) -> None:
        response = client.tests.with_raw_response.list_batch_tests(
            type="retell-llm",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_batch_tests(self, client: Retell) -> None:
        with client.tests.with_streaming_response.list_batch_tests(
            type="retell-llm",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_test_case_definitions(self, client: Retell) -> None:
        test = client.tests.list_test_case_definitions(
            type="retell-llm",
        )
        assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_test_case_definitions_with_all_params(self, client: Retell) -> None:
        test = client.tests.list_test_case_definitions(
            type="retell-llm",
            conversation_flow_id="conversation_flow_id",
            llm_id="llm_id",
            version=0,
        )
        assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_test_case_definitions(self, client: Retell) -> None:
        response = client.tests.with_raw_response.list_test_case_definitions(
            type="retell-llm",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_test_case_definitions(self, client: Retell) -> None:
        with client.tests.with_streaming_response.list_test_case_definitions(
            type="retell-llm",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_test_runs(self, client: Retell) -> None:
        test = client.tests.list_test_runs(
            "test_case_batch_job_id",
        )
        assert_matches_type(TestListTestRunsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_test_runs(self, client: Retell) -> None:
        response = client.tests.with_raw_response.list_test_runs(
            "test_case_batch_job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestListTestRunsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_test_runs(self, client: Retell) -> None:
        with client.tests.with_streaming_response.list_test_runs(
            "test_case_batch_job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestListTestRunsResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list_test_runs(self, client: Retell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_batch_job_id` but received ''"
        ):
            client.tests.with_raw_response.list_test_runs(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_test_case_definition(self, client: Retell) -> None:
        test = client.tests.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_test_case_definition_with_all_params(self, client: Retell) -> None:
        test = client.tests.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
            dynamic_variables={"foo": "string"},
            llm_model="gpt-4.1",
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
                "version": 0,
            },
            tool_mocks=[
                {
                    "input_match_rule": {"type": "any"},
                    "output": "output",
                    "tool_name": "tool_name",
                    "result": True,
                }
            ],
            user_prompt="user_prompt",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_test_case_definition(self, client: Retell) -> None:
        response = client.tests.with_raw_response.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = response.parse()
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_test_case_definition(self, client: Retell) -> None:
        with client.tests.with_streaming_response.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = response.parse()
            assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_test_case_definition(self, client: Retell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_definition_id` but received ''"
        ):
            client.tests.with_raw_response.update_test_case_definition(
                test_case_definition_id="",
            )


class TestAsyncTests:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_batch_test(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            test_case_definition_ids=["string"],
        )
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_batch_test_with_all_params(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
                "version": 0,
            },
            test_case_definition_ids=["string"],
        )
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_batch_test(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            test_case_definition_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_batch_test(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.create_batch_test(
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            test_case_definition_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(BatchTestResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_test_case_definition(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            user_prompt="user_prompt",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_test_case_definition_with_all_params(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
                "version": 0,
            },
            user_prompt="user_prompt",
            dynamic_variables={"foo": "string"},
            llm_model="gpt-4.1",
            tool_mocks=[
                {
                    "input_match_rule": {"type": "any"},
                    "output": "output",
                    "tool_name": "tool_name",
                    "result": True,
                }
            ],
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_test_case_definition(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            user_prompt="user_prompt",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_test_case_definition(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.create_test_case_definition(
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
            },
            user_prompt="user_prompt",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete_test_case_definition(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.delete_test_case_definition(
            "test_case_definition_id",
        )
        assert test is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete_test_case_definition(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.delete_test_case_definition(
            "test_case_definition_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert test is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete_test_case_definition(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.delete_test_case_definition(
            "test_case_definition_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert test is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete_test_case_definition(self, async_client: AsyncRetell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_definition_id` but received ''"
        ):
            await async_client.tests.with_raw_response.delete_test_case_definition(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_batch_test(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.get_batch_test(
            "test_case_batch_job_id",
        )
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_batch_test(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.get_batch_test(
            "test_case_batch_job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(BatchTestResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_batch_test(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.get_batch_test(
            "test_case_batch_job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(BatchTestResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_batch_test(self, async_client: AsyncRetell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_batch_job_id` but received ''"
        ):
            await async_client.tests.with_raw_response.get_batch_test(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_test_case_definition(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.get_test_case_definition(
            "test_case_definition_id",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_test_case_definition(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.get_test_case_definition(
            "test_case_definition_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_test_case_definition(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.get_test_case_definition(
            "test_case_definition_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_test_case_definition(self, async_client: AsyncRetell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_definition_id` but received ''"
        ):
            await async_client.tests.with_raw_response.get_test_case_definition(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_test_run(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.get_test_run(
            "test_case_job_id",
        )
        assert_matches_type(TestCaseJobResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_test_run(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.get_test_run(
            "test_case_job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestCaseJobResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_test_run(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.get_test_run(
            "test_case_job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestCaseJobResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_test_run(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_job_id` but received ''"):
            await async_client.tests.with_raw_response.get_test_run(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_batch_tests(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.list_batch_tests(
            type="retell-llm",
        )
        assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_batch_tests_with_all_params(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.list_batch_tests(
            type="retell-llm",
            conversation_flow_id="conversation_flow_id",
            llm_id="llm_id",
            version=0,
        )
        assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_batch_tests(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.list_batch_tests(
            type="retell-llm",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_batch_tests(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.list_batch_tests(
            type="retell-llm",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestListBatchTestsResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_test_case_definitions(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.list_test_case_definitions(
            type="retell-llm",
        )
        assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_test_case_definitions_with_all_params(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.list_test_case_definitions(
            type="retell-llm",
            conversation_flow_id="conversation_flow_id",
            llm_id="llm_id",
            version=0,
        )
        assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_test_case_definitions(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.list_test_case_definitions(
            type="retell-llm",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_test_case_definitions(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.list_test_case_definitions(
            type="retell-llm",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestListTestCaseDefinitionsResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_test_runs(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.list_test_runs(
            "test_case_batch_job_id",
        )
        assert_matches_type(TestListTestRunsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_test_runs(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.list_test_runs(
            "test_case_batch_job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestListTestRunsResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_test_runs(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.list_test_runs(
            "test_case_batch_job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestListTestRunsResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list_test_runs(self, async_client: AsyncRetell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_batch_job_id` but received ''"
        ):
            await async_client.tests.with_raw_response.list_test_runs(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_test_case_definition(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_test_case_definition_with_all_params(self, async_client: AsyncRetell) -> None:
        test = await async_client.tests.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
            dynamic_variables={"foo": "string"},
            llm_model="gpt-4.1",
            metrics=["string"],
            name="name",
            response_engine={
                "llm_id": "llm_id",
                "type": "retell-llm",
                "version": 0,
            },
            tool_mocks=[
                {
                    "input_match_rule": {"type": "any"},
                    "output": "output",
                    "tool_name": "tool_name",
                    "result": True,
                }
            ],
            user_prompt="user_prompt",
        )
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_test_case_definition(self, async_client: AsyncRetell) -> None:
        response = await async_client.tests.with_raw_response.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test = await response.parse()
        assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_test_case_definition(self, async_client: AsyncRetell) -> None:
        async with async_client.tests.with_streaming_response.update_test_case_definition(
            test_case_definition_id="test_case_definition_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test = await response.parse()
            assert_matches_type(TestCaseDefinitionResponse, test, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_test_case_definition(self, async_client: AsyncRetell) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `test_case_definition_id` but received ''"
        ):
            await async_client.tests.with_raw_response.update_test_case_definition(
                test_case_definition_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "ChatAgentUpdateParams",
    "GuardrailConfig",
    "HandbookConfig",
    "PiiConfig",
    "PostChatAnalysisData",
    "PostChatAnalysisDataStringAnalysisData",
    "PostChatAnalysisDataEnumAnalysisData",
    "PostChatAnalysisDataBooleanAnalysisData",
    "PostChatAnalysisDataNumberAnalysisData",
    "PostChatAnalysisDataChatPresetAnalysisData",
    "ResponseEngine",
    "ResponseEngineResponseEngineRetellLm",
    "ResponseEngineResponseEngineCustomLm",
    "ResponseEngineResponseEngineConversationFlow",
]


class ChatAgentUpdateParams(TypedDict, total=False):
    version: int
    """Optional version of the API to use for this request. Default to latest version."""

    agent_name: Optional[str]
    """The name of the chat agent. Only used for your own reference."""

    analysis_successful_prompt: Optional[str]
    """
    The prompt to use for post call analysis to evaluate whether the call is
    successful. Set to null to use the default prompt.
    """

    analysis_summary_prompt: Optional[str]
    """The prompt to use for post call analysis to summarize the call.

    Set to null to use the default prompt.
    """

    analysis_user_sentiment_prompt: Optional[str]
    """Prompt to guide how the post chat analysis should evaluate user sentiment.

    When unset, the default system prompt is used. Set to null to use the default
    prompt.
    """

    auto_close_message: Optional[str]
    """Message to display when the chat is automatically closed."""

    data_storage_retention_days: Optional[int]
    """Number of days to retain call/chat data before automatic deletion.

    Must be between 1 and 730 days. If not set, data is retained forever (no
    automatic deletion).
    """

    data_storage_setting: Optional[Literal["everything", "everything_except_pii", "basic_attributes_only"]]
    """Controls what data is stored for this agent.

    "everything" stores all data including transcripts and recordings.
    "everything_except_pii" stores data but excludes PII when possible based on PII
    configuration. "basic_attributes_only" stores only basic metadata. If not set,
    defaults to "everything".
    """

    end_chat_after_silence_ms: Optional[int]
    """If users stay silent for a period after agent speech, end the chat.

    The minimum value allowed is 120,000 ms (2 minutes). The maximum value allowed
    is 259,200,000 ms (72 hours). By default, this is set to 3,600,000 (1 hour).
    """

    guardrail_config: GuardrailConfig
    """
    Configuration for guardrail checks to detect and prevent prohibited topics in
    agent output and user input.
    """

    handbook_config: HandbookConfig
    """Toggle behavior presets on/off to influence agent response style and behaviors.

    Voice-only presets are not available for chat agents.
    """

    is_public: Optional[bool]
    """Whether the agent is public.

    When set to true, the agent is available for public agent preview link.
    """

    language: Literal[
        "en-US",
        "en-IN",
        "en-GB",
        "en-AU",
        "en-NZ",
        "de-DE",
        "es-ES",
        "es-419",
        "hi-IN",
        "fr-FR",
        "fr-CA",
        "ja-JP",
        "pt-PT",
        "pt-BR",
        "zh-CN",
        "ru-RU",
        "it-IT",
        "ko-KR",
        "nl-NL",
        "nl-BE",
        "pl-PL",
        "tr-TR",
        "th-TH",
        "vi-VN",
        "ro-RO",
        "bg-BG",
        "ca-ES",
        "da-DK",
        "fi-FI",
        "el-GR",
        "hu-HU",
        "id-ID",
        "no-NO",
        "sk-SK",
        "sv-SE",
        "lt-LT",
        "lv-LV",
        "cs-CZ",
        "multi",
    ]
    """Specifies what language (and dialect) the chat will operate in.

    For instance, selecting `en-GB` optimizes for British English. If unset, will
    use default value `en-US`. Select `multi` for multilingual support, currently
    this supports Spanish and English.
    """

    opt_in_signed_url: bool
    """Whether this agent opts in to signed url for public log.

    If not set, default value of false will apply.
    """

    pii_config: PiiConfig
    """Configuration for PII scrubbing from transcripts and recordings."""

    post_chat_analysis_data: Optional[Iterable[PostChatAnalysisData]]
    """Post chat analysis data to extract from the chat.

    This data will augment the pre-defined variables extracted in the chat analysis.
    This will be available after the chat ends.
    """

    post_chat_analysis_model: Optional[
        Literal[
            "gpt-4.1",
            "gpt-4.1-mini",
            "gpt-4.1-nano",
            "gpt-5",
            "gpt-5-mini",
            "gpt-5-nano",
            "gpt-5.1",
            "gpt-5.2",
            "gpt-5.4",
            "gpt-5.4-mini",
            "gpt-5.4-nano",
            "claude-4.5-sonnet",
            "claude-4.6-sonnet",
            "claude-4.5-haiku",
            "gemini-2.5-flash",
            "gemini-2.5-flash-lite",
            "gemini-3.0-flash",
        ]
    ]
    """The model to use for post chat analysis. Default to gpt-4.1-mini."""

    response_engine: ResponseEngine
    """The Response Engine to attach to the agent.

    It is used to generate responses for the agent. You need to create a Response
    Engine first before attaching it to an agent.
    """

    signed_url_expiration_ms: Optional[int]
    """The expiration time for the signed url in milliseconds.

    Only applicable when opt_in_signed_url is true. If not set, default value of
    86400000 (24 hours) will apply.
    """

    timezone: Optional[str]
    """IANA timezone for the agent (e.g.

    America/New_York). Defaults to America/Los_Angeles if not set.
    """

    webhook_events: Optional[List[Literal["chat_started", "chat_ended", "chat_analyzed"]]]
    """Which webhook events this agent should receive.

    If not set, defaults to chat_started, chat_ended, chat_analyzed.
    """

    webhook_timeout_ms: int
    """The timeout for the webhook in milliseconds.

    If not set, default value of 10000 will apply.
    """

    webhook_url: Optional[str]
    """The webhook for agent to listen to chat events.

    See what events it would get at [webhook doc](/features/webhook). If set, will
    binds webhook events for this agent to the specified url, and will ignore the
    account level webhook for this agent. Set to `null` to remove webhook url from
    this agent.
    """


class GuardrailConfig(TypedDict, total=False):
    """
    Configuration for guardrail checks to detect and prevent prohibited topics in agent output and user input.
    """

    input_topics: Optional[List[Literal["platform_integrity_jailbreaking"]]]
    """Selected prohibited user topic categories to check.

    When user messages contain these topics, the agent will respond with a
    placeholder message instead of processing the request.
    """

    output_topics: Optional[
        List[
            Literal[
                "harassment",
                "self_harm",
                "sexual_exploitation",
                "violence",
                "defense_and_national_security",
                "illicit_and_harmful_activity",
                "gambling",
                "regulated_professional_advice",
                "child_safety_and_exploitation",
            ]
        ]
    ]
    """Selected prohibited agent topic categories to check.

    When agent messages contain these topics, they will be replaced with a
    placeholder message.
    """


class HandbookConfig(TypedDict, total=False):
    """Toggle behavior presets on/off to influence agent response style and behaviors.

    Voice-only presets are not available for chat agents.
    """

    ai_disclosure: bool
    """When asked, acknowledge being a virtual assistant."""

    default_personality: bool
    """Professional call center rep baseline."""

    high_empathy: bool
    """Warm acknowledgment of caller concerns."""

    scope_boundaries: bool
    """Stay within prompt/context scope, don't invent details."""


class PiiConfig(TypedDict, total=False):
    """Configuration for PII scrubbing from transcripts and recordings."""

    categories: Required[
        List[
            Literal[
                "person_name",
                "address",
                "email",
                "phone_number",
                "ssn",
                "passport",
                "driver_license",
                "credit_card",
                "bank_account",
                "password",
                "pin",
                "medical_id",
                "date_of_birth",
                "customer_account_number",
            ]
        ]
    ]
    """List of PII categories to scrub from transcripts and recordings."""

    mode: Required[Literal["post_call"]]
    """The processing mode for PII scrubbing. Currently only post-call is supported."""


class PostChatAnalysisDataStringAnalysisData(TypedDict, total=False):
    description: Required[str]
    """Description of the variable."""

    name: Required[str]
    """Name of the variable."""

    type: Required[Literal["string"]]
    """Type of the variable to extract."""

    conditional_prompt: str
    """
    Optional instruction to help decide whether this field needs to be populated in
    the analysis. If not set, the field is always included. If required is true,
    this is ignored.
    """

    examples: SequenceNotStr[str]
    """Examples of the variable value to teach model the style and syntax."""

    required: bool
    """Whether this data is required.

    If true and the data is not extracted, the call will be marked as unsuccessful.
    """


class PostChatAnalysisDataEnumAnalysisData(TypedDict, total=False):
    choices: Required[SequenceNotStr[str]]
    """The possible values of the variable, must be non empty array."""

    description: Required[str]
    """Description of the variable."""

    name: Required[str]
    """Name of the variable."""

    type: Required[Literal["enum"]]
    """Type of the variable to extract."""

    conditional_prompt: str
    """
    Optional instruction to help decide whether this field needs to be populated in
    the analysis. If not set, the field is always included. If required is true,
    this is ignored.
    """

    required: bool
    """Whether this data is required.

    If true and the data is not extracted, the call will be marked as unsuccessful.
    """


class PostChatAnalysisDataBooleanAnalysisData(TypedDict, total=False):
    description: Required[str]
    """Description of the variable."""

    name: Required[str]
    """Name of the variable."""

    type: Required[Literal["boolean"]]
    """Type of the variable to extract."""

    conditional_prompt: str
    """
    Optional instruction to help decide whether this field needs to be populated in
    the analysis. If not set, the field is always included. If required is true,
    this is ignored.
    """

    required: bool
    """Whether this data is required.

    If true and the data is not extracted, the call will be marked as unsuccessful.
    """


class PostChatAnalysisDataNumberAnalysisData(TypedDict, total=False):
    description: Required[str]
    """Description of the variable."""

    name: Required[str]
    """Name of the variable."""

    type: Required[Literal["number"]]
    """Type of the variable to extract."""

    conditional_prompt: str
    """
    Optional instruction to help decide whether this field needs to be populated in
    the analysis. If not set, the field is always included. If required is true,
    this is ignored.
    """

    required: bool
    """Whether this data is required.

    If true and the data is not extracted, the call will be marked as unsuccessful.
    """


class PostChatAnalysisDataChatPresetAnalysisData(TypedDict, total=False):
    """System preset for post-chat analysis (chat agents).

    Use in post_chat_analysis_data to override prompts or mark fields optional.
    """

    name: Required[Literal["chat_summary", "chat_successful", "user_sentiment"]]
    """Preset identifier for chat agent analysis."""

    type: Required[Literal["system-presets"]]
    """Identifies this item as a system preset."""

    conditional_prompt: str
    """Optional instruction to help decide whether this field needs to be populated.

    If not set, the field is always included.
    """

    description: str
    """Prompt or description for this preset."""

    required: bool
    """If false, this field is optional in the analysis.

    If true or unset, the field is required.
    """


PostChatAnalysisData: TypeAlias = Union[
    PostChatAnalysisDataStringAnalysisData,
    PostChatAnalysisDataEnumAnalysisData,
    PostChatAnalysisDataBooleanAnalysisData,
    PostChatAnalysisDataNumberAnalysisData,
    PostChatAnalysisDataChatPresetAnalysisData,
]


class ResponseEngineResponseEngineRetellLm(TypedDict, total=False):
    llm_id: Required[str]
    """id of the Retell LLM Response Engine."""

    type: Required[Literal["retell-llm"]]
    """type of the Response Engine."""

    version: Optional[float]
    """Version of the Retell LLM Response Engine."""


class ResponseEngineResponseEngineCustomLm(TypedDict, total=False):
    llm_websocket_url: Required[str]
    """LLM websocket url of the custom LLM."""

    type: Required[Literal["custom-llm"]]
    """type of the Response Engine."""


class ResponseEngineResponseEngineConversationFlow(TypedDict, total=False):
    conversation_flow_id: Required[str]
    """ID of the Conversation Flow Response Engine."""

    type: Required[Literal["conversation-flow"]]
    """type of the Response Engine."""

    version: Optional[float]
    """Version of the Conversation Flow Response Engine."""


ResponseEngine: TypeAlias = Union[
    ResponseEngineResponseEngineRetellLm,
    ResponseEngineResponseEngineCustomLm,
    ResponseEngineResponseEngineConversationFlow,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from retell import Retell, AsyncRetell
from tests.utils import assert_matches_type
from retell.types import (
    ConversationFlowResponse,
    ConversationFlowListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConversationFlow:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                }
            ],
            start_speaker="agent",
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
                "high_priority": True,
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                    "always_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "prompt",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "display_position": {
                        "x": 0,
                        "y": 0,
                    },
                    "edges": [
                        {
                            "id": "edge_1",
                            "transition_condition": {
                                "prompt": "Customer wants to book appointment",
                                "type": "prompt",
                            },
                            "destination_node_id": "book_appointment",
                        }
                    ],
                    "finetune_conversation_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                        }
                    ],
                    "finetune_transition_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                            "destination_node_id": "destination_node_id",
                        }
                    ],
                    "global_node_setting": {
                        "condition": "condition",
                        "cool_down": 1,
                        "go_back_conditions": [
                            {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            }
                        ],
                        "negative_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                        "positive_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                    },
                    "interruption_sensitivity": 0,
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-4.1",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "name": "name",
                    "responsiveness": 0,
                    "skip_response_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "Skip response",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "voice_speed": 0.5,
                }
            ],
            start_speaker="agent",
            begin_after_user_silence_ms=2000,
            begin_tag_display_position={
                "x": 100,
                "y": 200,
            },
            components=[
                {
                    "name": "Customer Information Collector",
                    "nodes": [
                        {
                            "id": "collect_info",
                            "instruction": {
                                "text": "Ask the customer for their name and contact information.",
                                "type": "prompt",
                            },
                            "type": "conversation",
                            "always_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "display_position": {
                                "x": 0,
                                "y": 0,
                            },
                            "edges": [
                                {
                                    "id": "id",
                                    "transition_condition": {
                                        "prompt": "prompt",
                                        "type": "prompt",
                                    },
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "finetune_conversation_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                }
                            ],
                            "finetune_transition_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "global_node_setting": {
                                "condition": "condition",
                                "cool_down": 1,
                                "go_back_conditions": [
                                    {
                                        "id": "id",
                                        "transition_condition": {
                                            "prompt": "prompt",
                                            "type": "prompt",
                                        },
                                        "destination_node_id": "destination_node_id",
                                    }
                                ],
                                "negative_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                                "positive_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                            },
                            "interruption_sensitivity": 0,
                            "knowledge_base_ids": ["kb_001", "kb_002"],
                            "model_choice": {
                                "model": "gpt-4.1",
                                "type": "cascading",
                                "high_priority": True,
                            },
                            "name": "name",
                            "responsiveness": 0,
                            "skip_response_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "voice_speed": 0.5,
                        }
                    ],
                    "begin_tag_display_position": {
                        "x": 100,
                        "y": 200,
                    },
                    "mcps": [
                        {
                            "name": "name",
                            "url": "url",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "query_params": {
                                "index": "1",
                                "key": "value",
                            },
                            "timeout_ms": 0,
                        }
                    ],
                    "notes": [
                        {
                            "id": "note_abc123",
                            "content": "Remember to handle edge cases here.",
                            "display_position": {
                                "x": 300,
                                "y": 150,
                            },
                            "size": {
                                "height": 100,
                                "width": 200,
                            },
                        }
                    ],
                    "start_node_id": "collect_info",
                    "tools": [
                        {
                            "name": "get_customer_info",
                            "type": "custom",
                            "url": "https://api.example.com/customer",
                            "args_at_root": True,
                            "description": "Get customer information from database",
                            "execution_message_description": "execution_message_description",
                            "execution_message_type": "prompt",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "method": "GET",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                            "query_params": {
                                "page": "1",
                                "sort": "asc",
                            },
                            "response_variables": {"user_name": "data.user.name"},
                            "speak_after_execution": True,
                            "speak_during_execution": True,
                            "timeout_ms": 0,
                            "tool_id": "tool_001",
                        }
                    ],
                }
            ],
            default_dynamic_variables={
                "company_name": "Retell Inc",
                "support_hours": "9 AM - 5 PM",
            },
            global_prompt="You are a helpful customer service agent.",
            is_transfer_llm=False,
            kb_config={
                "filter_score": 0.6,
                "top_k": 3,
            },
            knowledge_base_ids=["kb_001", "kb_002"],
            mcps=[
                {
                    "name": "name",
                    "url": "url",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "query_params": {
                        "index": "1",
                        "key": "value",
                    },
                    "timeout_ms": 0,
                }
            ],
            model_temperature=0.7,
            notes=[
                {
                    "id": "note_abc123",
                    "content": "Remember to handle edge cases here.",
                    "display_position": {
                        "x": 300,
                        "y": 150,
                    },
                    "size": {
                        "height": 100,
                        "width": 200,
                    },
                }
            ],
            start_node_id="start",
            tool_call_strict_mode=True,
            tools=[
                {
                    "name": "get_customer_info",
                    "type": "custom",
                    "url": "https://api.example.com/customer",
                    "args_at_root": True,
                    "description": "Get customer information from database",
                    "execution_message_description": "execution_message_description",
                    "execution_message_type": "prompt",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "method": "GET",
                    "parameters": {
                        "properties": {"foo": "bar"},
                        "type": "object",
                        "required": ["string"],
                    },
                    "query_params": {
                        "page": "1",
                        "sort": "asc",
                    },
                    "response_variables": {"user_name": "data.user.name"},
                    "speak_after_execution": True,
                    "speak_during_execution": True,
                    "timeout_ms": 0,
                    "tool_id": "tool_001",
                }
            ],
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Retell) -> None:
        response = client.conversation_flow.with_raw_response.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                }
            ],
            start_speaker="agent",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = response.parse()
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Retell) -> None:
        with client.conversation_flow.with_streaming_response.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                }
            ],
            start_speaker="agent",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = response.parse()
            assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.retrieve(
            conversation_flow_id="conversation_flow_id",
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.retrieve(
            conversation_flow_id="conversation_flow_id",
            version=1,
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Retell) -> None:
        response = client.conversation_flow.with_raw_response.retrieve(
            conversation_flow_id="conversation_flow_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = response.parse()
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Retell) -> None:
        with client.conversation_flow.with_streaming_response.retrieve(
            conversation_flow_id="conversation_flow_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = response.parse()
            assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_flow_id` but received ''"):
            client.conversation_flow.with_raw_response.retrieve(
                conversation_flow_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.update(
            conversation_flow_id="conversation_flow_id",
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.update(
            conversation_flow_id="conversation_flow_id",
            version=1,
            begin_after_user_silence_ms=2000,
            begin_tag_display_position={
                "x": 100,
                "y": 200,
            },
            components=[
                {
                    "name": "Customer Information Collector",
                    "nodes": [
                        {
                            "id": "collect_info",
                            "instruction": {
                                "text": "Ask the customer for their name and contact information.",
                                "type": "prompt",
                            },
                            "type": "conversation",
                            "always_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "display_position": {
                                "x": 0,
                                "y": 0,
                            },
                            "edges": [
                                {
                                    "id": "id",
                                    "transition_condition": {
                                        "prompt": "prompt",
                                        "type": "prompt",
                                    },
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "finetune_conversation_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                }
                            ],
                            "finetune_transition_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "global_node_setting": {
                                "condition": "condition",
                                "cool_down": 1,
                                "go_back_conditions": [
                                    {
                                        "id": "id",
                                        "transition_condition": {
                                            "prompt": "prompt",
                                            "type": "prompt",
                                        },
                                        "destination_node_id": "destination_node_id",
                                    }
                                ],
                                "negative_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                                "positive_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                            },
                            "interruption_sensitivity": 0,
                            "knowledge_base_ids": ["kb_001", "kb_002"],
                            "model_choice": {
                                "model": "gpt-4.1",
                                "type": "cascading",
                                "high_priority": True,
                            },
                            "name": "name",
                            "responsiveness": 0,
                            "skip_response_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "voice_speed": 0.5,
                        }
                    ],
                    "begin_tag_display_position": {
                        "x": 100,
                        "y": 200,
                    },
                    "mcps": [
                        {
                            "name": "name",
                            "url": "url",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "query_params": {
                                "index": "1",
                                "key": "value",
                            },
                            "timeout_ms": 0,
                        }
                    ],
                    "notes": [
                        {
                            "id": "note_abc123",
                            "content": "Remember to handle edge cases here.",
                            "display_position": {
                                "x": 300,
                                "y": 150,
                            },
                            "size": {
                                "height": 100,
                                "width": 200,
                            },
                        }
                    ],
                    "start_node_id": "collect_info",
                    "tools": [
                        {
                            "name": "get_customer_info",
                            "type": "custom",
                            "url": "https://api.example.com/customer",
                            "args_at_root": True,
                            "description": "Get customer information from database",
                            "execution_message_description": "execution_message_description",
                            "execution_message_type": "prompt",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "method": "GET",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                            "query_params": {
                                "page": "1",
                                "sort": "asc",
                            },
                            "response_variables": {"user_name": "data.user.name"},
                            "speak_after_execution": True,
                            "speak_during_execution": True,
                            "timeout_ms": 0,
                            "tool_id": "tool_001",
                        }
                    ],
                }
            ],
            default_dynamic_variables={
                "company_name": "Retell Inc",
                "support_hours": "9 AM - 5 PM",
            },
            global_prompt="You are a helpful customer service agent.",
            is_transfer_llm=False,
            kb_config={
                "filter_score": 0.6,
                "top_k": 3,
            },
            knowledge_base_ids=["kb_001", "kb_002"],
            mcps=[
                {
                    "name": "name",
                    "url": "url",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "query_params": {
                        "index": "1",
                        "key": "value",
                    },
                    "timeout_ms": 0,
                }
            ],
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
                "high_priority": True,
            },
            model_temperature=0.7,
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                    "always_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "prompt",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "display_position": {
                        "x": 0,
                        "y": 0,
                    },
                    "edges": [
                        {
                            "id": "edge_1",
                            "transition_condition": {
                                "prompt": "Customer wants to book appointment",
                                "type": "prompt",
                            },
                            "destination_node_id": "book_appointment",
                        }
                    ],
                    "finetune_conversation_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                        }
                    ],
                    "finetune_transition_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                            "destination_node_id": "destination_node_id",
                        }
                    ],
                    "global_node_setting": {
                        "condition": "condition",
                        "cool_down": 1,
                        "go_back_conditions": [
                            {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            }
                        ],
                        "negative_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                        "positive_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                    },
                    "interruption_sensitivity": 0,
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-4.1",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "name": "name",
                    "responsiveness": 0,
                    "skip_response_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "Skip response",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "voice_speed": 0.5,
                }
            ],
            notes=[
                {
                    "id": "note_abc123",
                    "content": "Remember to handle edge cases here.",
                    "display_position": {
                        "x": 300,
                        "y": 150,
                    },
                    "size": {
                        "height": 100,
                        "width": 200,
                    },
                }
            ],
            start_node_id="start",
            start_speaker="agent",
            tool_call_strict_mode=True,
            tools=[
                {
                    "name": "get_customer_info",
                    "type": "custom",
                    "url": "https://api.example.com/customer",
                    "args_at_root": True,
                    "description": "Get customer information from database",
                    "execution_message_description": "execution_message_description",
                    "execution_message_type": "prompt",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "method": "GET",
                    "parameters": {
                        "properties": {"foo": "bar"},
                        "type": "object",
                        "required": ["string"],
                    },
                    "query_params": {
                        "page": "1",
                        "sort": "asc",
                    },
                    "response_variables": {"user_name": "data.user.name"},
                    "speak_after_execution": True,
                    "speak_during_execution": True,
                    "timeout_ms": 0,
                    "tool_id": "tool_001",
                }
            ],
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Retell) -> None:
        response = client.conversation_flow.with_raw_response.update(
            conversation_flow_id="conversation_flow_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = response.parse()
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Retell) -> None:
        with client.conversation_flow.with_streaming_response.update(
            conversation_flow_id="conversation_flow_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = response.parse()
            assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_flow_id` but received ''"):
            client.conversation_flow.with_raw_response.update(
                conversation_flow_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.list()
        assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.list(
            limit=1000,
            pagination_key="pagination_key",
            pagination_key_version=0,
        )
        assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Retell) -> None:
        response = client.conversation_flow.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = response.parse()
        assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Retell) -> None:
        with client.conversation_flow.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = response.parse()
            assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Retell) -> None:
        conversation_flow = client.conversation_flow.delete(
            "conversation_flow_id",
        )
        assert conversation_flow is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Retell) -> None:
        response = client.conversation_flow.with_raw_response.delete(
            "conversation_flow_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = response.parse()
        assert conversation_flow is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Retell) -> None:
        with client.conversation_flow.with_streaming_response.delete(
            "conversation_flow_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = response.parse()
            assert conversation_flow is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Retell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_flow_id` but received ''"):
            client.conversation_flow.with_raw_response.delete(
                "",
            )


class TestAsyncConversationFlow:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                }
            ],
            start_speaker="agent",
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
                "high_priority": True,
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                    "always_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "prompt",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "display_position": {
                        "x": 0,
                        "y": 0,
                    },
                    "edges": [
                        {
                            "id": "edge_1",
                            "transition_condition": {
                                "prompt": "Customer wants to book appointment",
                                "type": "prompt",
                            },
                            "destination_node_id": "book_appointment",
                        }
                    ],
                    "finetune_conversation_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                        }
                    ],
                    "finetune_transition_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                            "destination_node_id": "destination_node_id",
                        }
                    ],
                    "global_node_setting": {
                        "condition": "condition",
                        "cool_down": 1,
                        "go_back_conditions": [
                            {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            }
                        ],
                        "negative_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                        "positive_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                    },
                    "interruption_sensitivity": 0,
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-4.1",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "name": "name",
                    "responsiveness": 0,
                    "skip_response_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "Skip response",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "voice_speed": 0.5,
                }
            ],
            start_speaker="agent",
            begin_after_user_silence_ms=2000,
            begin_tag_display_position={
                "x": 100,
                "y": 200,
            },
            components=[
                {
                    "name": "Customer Information Collector",
                    "nodes": [
                        {
                            "id": "collect_info",
                            "instruction": {
                                "text": "Ask the customer for their name and contact information.",
                                "type": "prompt",
                            },
                            "type": "conversation",
                            "always_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "display_position": {
                                "x": 0,
                                "y": 0,
                            },
                            "edges": [
                                {
                                    "id": "id",
                                    "transition_condition": {
                                        "prompt": "prompt",
                                        "type": "prompt",
                                    },
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "finetune_conversation_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                }
                            ],
                            "finetune_transition_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "global_node_setting": {
                                "condition": "condition",
                                "cool_down": 1,
                                "go_back_conditions": [
                                    {
                                        "id": "id",
                                        "transition_condition": {
                                            "prompt": "prompt",
                                            "type": "prompt",
                                        },
                                        "destination_node_id": "destination_node_id",
                                    }
                                ],
                                "negative_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                                "positive_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                            },
                            "interruption_sensitivity": 0,
                            "knowledge_base_ids": ["kb_001", "kb_002"],
                            "model_choice": {
                                "model": "gpt-4.1",
                                "type": "cascading",
                                "high_priority": True,
                            },
                            "name": "name",
                            "responsiveness": 0,
                            "skip_response_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "voice_speed": 0.5,
                        }
                    ],
                    "begin_tag_display_position": {
                        "x": 100,
                        "y": 200,
                    },
                    "mcps": [
                        {
                            "name": "name",
                            "url": "url",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "query_params": {
                                "index": "1",
                                "key": "value",
                            },
                            "timeout_ms": 0,
                        }
                    ],
                    "notes": [
                        {
                            "id": "note_abc123",
                            "content": "Remember to handle edge cases here.",
                            "display_position": {
                                "x": 300,
                                "y": 150,
                            },
                            "size": {
                                "height": 100,
                                "width": 200,
                            },
                        }
                    ],
                    "start_node_id": "collect_info",
                    "tools": [
                        {
                            "name": "get_customer_info",
                            "type": "custom",
                            "url": "https://api.example.com/customer",
                            "args_at_root": True,
                            "description": "Get customer information from database",
                            "execution_message_description": "execution_message_description",
                            "execution_message_type": "prompt",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "method": "GET",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                            "query_params": {
                                "page": "1",
                                "sort": "asc",
                            },
                            "response_variables": {"user_name": "data.user.name"},
                            "speak_after_execution": True,
                            "speak_during_execution": True,
                            "timeout_ms": 0,
                            "tool_id": "tool_001",
                        }
                    ],
                }
            ],
            default_dynamic_variables={
                "company_name": "Retell Inc",
                "support_hours": "9 AM - 5 PM",
            },
            global_prompt="You are a helpful customer service agent.",
            is_transfer_llm=False,
            kb_config={
                "filter_score": 0.6,
                "top_k": 3,
            },
            knowledge_base_ids=["kb_001", "kb_002"],
            mcps=[
                {
                    "name": "name",
                    "url": "url",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "query_params": {
                        "index": "1",
                        "key": "value",
                    },
                    "timeout_ms": 0,
                }
            ],
            model_temperature=0.7,
            notes=[
                {
                    "id": "note_abc123",
                    "content": "Remember to handle edge cases here.",
                    "display_position": {
                        "x": 300,
                        "y": 150,
                    },
                    "size": {
                        "height": 100,
                        "width": 200,
                    },
                }
            ],
            start_node_id="start",
            tool_call_strict_mode=True,
            tools=[
                {
                    "name": "get_customer_info",
                    "type": "custom",
                    "url": "https://api.example.com/customer",
                    "args_at_root": True,
                    "description": "Get customer information from database",
                    "execution_message_description": "execution_message_description",
                    "execution_message_type": "prompt",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "method": "GET",
                    "parameters": {
                        "properties": {"foo": "bar"},
                        "type": "object",
                        "required": ["string"],
                    },
                    "query_params": {
                        "page": "1",
                        "sort": "asc",
                    },
                    "response_variables": {"user_name": "data.user.name"},
                    "speak_after_execution": True,
                    "speak_during_execution": True,
                    "timeout_ms": 0,
                    "tool_id": "tool_001",
                }
            ],
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRetell) -> None:
        response = await async_client.conversation_flow.with_raw_response.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                }
            ],
            start_speaker="agent",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = await response.parse()
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRetell) -> None:
        async with async_client.conversation_flow.with_streaming_response.create(
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
            },
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                }
            ],
            start_speaker="agent",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = await response.parse()
            assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.retrieve(
            conversation_flow_id="conversation_flow_id",
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.retrieve(
            conversation_flow_id="conversation_flow_id",
            version=1,
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRetell) -> None:
        response = await async_client.conversation_flow.with_raw_response.retrieve(
            conversation_flow_id="conversation_flow_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = await response.parse()
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRetell) -> None:
        async with async_client.conversation_flow.with_streaming_response.retrieve(
            conversation_flow_id="conversation_flow_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = await response.parse()
            assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_flow_id` but received ''"):
            await async_client.conversation_flow.with_raw_response.retrieve(
                conversation_flow_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.update(
            conversation_flow_id="conversation_flow_id",
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.update(
            conversation_flow_id="conversation_flow_id",
            version=1,
            begin_after_user_silence_ms=2000,
            begin_tag_display_position={
                "x": 100,
                "y": 200,
            },
            components=[
                {
                    "name": "Customer Information Collector",
                    "nodes": [
                        {
                            "id": "collect_info",
                            "instruction": {
                                "text": "Ask the customer for their name and contact information.",
                                "type": "prompt",
                            },
                            "type": "conversation",
                            "always_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "display_position": {
                                "x": 0,
                                "y": 0,
                            },
                            "edges": [
                                {
                                    "id": "id",
                                    "transition_condition": {
                                        "prompt": "prompt",
                                        "type": "prompt",
                                    },
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "finetune_conversation_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                }
                            ],
                            "finetune_transition_examples": [
                                {
                                    "id": "id",
                                    "transcript": [
                                        {
                                            "content": "content",
                                            "role": "agent",
                                        }
                                    ],
                                    "destination_node_id": "destination_node_id",
                                }
                            ],
                            "global_node_setting": {
                                "condition": "condition",
                                "cool_down": 1,
                                "go_back_conditions": [
                                    {
                                        "id": "id",
                                        "transition_condition": {
                                            "prompt": "prompt",
                                            "type": "prompt",
                                        },
                                        "destination_node_id": "destination_node_id",
                                    }
                                ],
                                "negative_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                                "positive_finetune_examples": [
                                    {
                                        "transcript": [
                                            {
                                                "content": "content",
                                                "role": "agent",
                                            }
                                        ]
                                    }
                                ],
                            },
                            "interruption_sensitivity": 0,
                            "knowledge_base_ids": ["kb_001", "kb_002"],
                            "model_choice": {
                                "model": "gpt-4.1",
                                "type": "cascading",
                                "high_priority": True,
                            },
                            "name": "name",
                            "responsiveness": 0,
                            "skip_response_edge": {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            },
                            "voice_speed": 0.5,
                        }
                    ],
                    "begin_tag_display_position": {
                        "x": 100,
                        "y": 200,
                    },
                    "mcps": [
                        {
                            "name": "name",
                            "url": "url",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "query_params": {
                                "index": "1",
                                "key": "value",
                            },
                            "timeout_ms": 0,
                        }
                    ],
                    "notes": [
                        {
                            "id": "note_abc123",
                            "content": "Remember to handle edge cases here.",
                            "display_position": {
                                "x": 300,
                                "y": 150,
                            },
                            "size": {
                                "height": 100,
                                "width": 200,
                            },
                        }
                    ],
                    "start_node_id": "collect_info",
                    "tools": [
                        {
                            "name": "get_customer_info",
                            "type": "custom",
                            "url": "https://api.example.com/customer",
                            "args_at_root": True,
                            "description": "Get customer information from database",
                            "execution_message_description": "execution_message_description",
                            "execution_message_type": "prompt",
                            "headers": {"Authorization": "Bearer 1234567890"},
                            "method": "GET",
                            "parameters": {
                                "properties": {"foo": "bar"},
                                "type": "object",
                                "required": ["string"],
                            },
                            "query_params": {
                                "page": "1",
                                "sort": "asc",
                            },
                            "response_variables": {"user_name": "data.user.name"},
                            "speak_after_execution": True,
                            "speak_during_execution": True,
                            "timeout_ms": 0,
                            "tool_id": "tool_001",
                        }
                    ],
                }
            ],
            default_dynamic_variables={
                "company_name": "Retell Inc",
                "support_hours": "9 AM - 5 PM",
            },
            global_prompt="You are a helpful customer service agent.",
            is_transfer_llm=False,
            kb_config={
                "filter_score": 0.6,
                "top_k": 3,
            },
            knowledge_base_ids=["kb_001", "kb_002"],
            mcps=[
                {
                    "name": "name",
                    "url": "url",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "query_params": {
                        "index": "1",
                        "key": "value",
                    },
                    "timeout_ms": 0,
                }
            ],
            model_choice={
                "model": "gpt-4.1",
                "type": "cascading",
                "high_priority": True,
            },
            model_temperature=0.7,
            nodes=[
                {
                    "id": "start",
                    "instruction": {
                        "text": "Greet the customer and ask how you can help them.",
                        "type": "prompt",
                    },
                    "type": "conversation",
                    "always_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "prompt",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "display_position": {
                        "x": 0,
                        "y": 0,
                    },
                    "edges": [
                        {
                            "id": "edge_1",
                            "transition_condition": {
                                "prompt": "Customer wants to book appointment",
                                "type": "prompt",
                            },
                            "destination_node_id": "book_appointment",
                        }
                    ],
                    "finetune_conversation_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                        }
                    ],
                    "finetune_transition_examples": [
                        {
                            "id": "id",
                            "transcript": [
                                {
                                    "content": "content",
                                    "role": "agent",
                                }
                            ],
                            "destination_node_id": "destination_node_id",
                        }
                    ],
                    "global_node_setting": {
                        "condition": "condition",
                        "cool_down": 1,
                        "go_back_conditions": [
                            {
                                "id": "id",
                                "transition_condition": {
                                    "prompt": "prompt",
                                    "type": "prompt",
                                },
                                "destination_node_id": "destination_node_id",
                            }
                        ],
                        "negative_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                        "positive_finetune_examples": [
                            {
                                "transcript": [
                                    {
                                        "content": "content",
                                        "role": "agent",
                                    }
                                ]
                            }
                        ],
                    },
                    "interruption_sensitivity": 0,
                    "knowledge_base_ids": ["kb_001", "kb_002"],
                    "model_choice": {
                        "model": "gpt-4.1",
                        "type": "cascading",
                        "high_priority": True,
                    },
                    "name": "name",
                    "responsiveness": 0,
                    "skip_response_edge": {
                        "id": "id",
                        "transition_condition": {
                            "prompt": "Skip response",
                            "type": "prompt",
                        },
                        "destination_node_id": "destination_node_id",
                    },
                    "voice_speed": 0.5,
                }
            ],
            notes=[
                {
                    "id": "note_abc123",
                    "content": "Remember to handle edge cases here.",
                    "display_position": {
                        "x": 300,
                        "y": 150,
                    },
                    "size": {
                        "height": 100,
                        "width": 200,
                    },
                }
            ],
            start_node_id="start",
            start_speaker="agent",
            tool_call_strict_mode=True,
            tools=[
                {
                    "name": "get_customer_info",
                    "type": "custom",
                    "url": "https://api.example.com/customer",
                    "args_at_root": True,
                    "description": "Get customer information from database",
                    "execution_message_description": "execution_message_description",
                    "execution_message_type": "prompt",
                    "headers": {"Authorization": "Bearer 1234567890"},
                    "method": "GET",
                    "parameters": {
                        "properties": {"foo": "bar"},
                        "type": "object",
                        "required": ["string"],
                    },
                    "query_params": {
                        "page": "1",
                        "sort": "asc",
                    },
                    "response_variables": {"user_name": "data.user.name"},
                    "speak_after_execution": True,
                    "speak_during_execution": True,
                    "timeout_ms": 0,
                    "tool_id": "tool_001",
                }
            ],
        )
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRetell) -> None:
        response = await async_client.conversation_flow.with_raw_response.update(
            conversation_flow_id="conversation_flow_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = await response.parse()
        assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRetell) -> None:
        async with async_client.conversation_flow.with_streaming_response.update(
            conversation_flow_id="conversation_flow_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = await response.parse()
            assert_matches_type(ConversationFlowResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_flow_id` but received ''"):
            await async_client.conversation_flow.with_raw_response.update(
                conversation_flow_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.list()
        assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.list(
            limit=1000,
            pagination_key="pagination_key",
            pagination_key_version=0,
        )
        assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRetell) -> None:
        response = await async_client.conversation_flow.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = await response.parse()
        assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRetell) -> None:
        async with async_client.conversation_flow.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = await response.parse()
            assert_matches_type(ConversationFlowListResponse, conversation_flow, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncRetell) -> None:
        conversation_flow = await async_client.conversation_flow.delete(
            "conversation_flow_id",
        )
        assert conversation_flow is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncRetell) -> None:
        response = await async_client.conversation_flow.with_raw_response.delete(
            "conversation_flow_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        conversation_flow = await response.parse()
        assert conversation_flow is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncRetell) -> None:
        async with async_client.conversation_flow.with_streaming_response.delete(
            "conversation_flow_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            conversation_flow = await response.parse()
            assert conversation_flow is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncRetell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_flow_id` but received ''"):
            await async_client.conversation_flow.with_raw_response.delete(
                "",
            )
